// aavsomat -- automatic measurement and reporting of observations for aavso
// Copyright (c) 2003 Alexandru Dan Corlan (http://dan.corlan.net)
//

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "../ccd/ccd.h"
#include "libnut.h"

// libccd variables

static struct sources src;
static struct ccd_frame *fr = NULL;
static struct vs_recipy vs;

// other local variables

static float xs[10000], ys[10000],
  ms[10000],     // guide magnitudes
  phms[10000], bt_phms[10000];   // magnitudes in the photometry band
static float catxs[10000], catys[10000], catms[10000], catfgms[10000], catjgms[10000];
static float catpx[10000], catpy[10000];
static int matches[10000];
static float romega, rdx, rdy, rampx, rampy, rdmag;
static  int nc,ns,nr;

// data structure for performing photometry

float targetx, targety;
static int varstars[1000];
int  refids[100];
float refx[10], refy[10], refm[10];
float measm[10];
int nrefs = 0;

char objn[30];
char obji[30];
char objc[30];
char obsc[30] = "ZZZ";
char chrt[30];

int verbosef;

void sketch_match(char *fn,int xsz,int ysz,int varst, char *catname) {
  FILE *epsf;
  int width=360;
  int height=245;
  float scalef = 0.3;
  float theta0, theta1;
  float dd, xx, yy, vx, vy;
  int i,j;

  char label[128] = "?";

  epsf = fopen(fn,"w");
  if (epsf==NULL) {
    fprintf(stderr,"Can't open %s\n", epsf);
    exit(-1);
  }
  fprintf(epsf, "%%!PS-Adobe-2.0 EPSF-3.0\n");
  fprintf(epsf, "%%%%Creator: (AVSOMAT V1.0 by A.D.Corlan MD)\n");
  fprintf(epsf, "%%%%BoundingBox: 0 0 %d %d\n\n", width, height);
  fprintf(epsf, "/Helvetica findfont 8 scalefont setfont\n");
  fprintf(epsf, "/picstar { %% x y radius ---\n");
  fprintf(epsf, "  /r exch def /y exch def /x exch def\n");
  fprintf(epsf, "  newpath x r add y moveto x y r 0 359 arc closepath fill } def\n");
  fprintf(epsf, "/catstar { %% (label) x y radius ---\n");
  fprintf(epsf, "  0.5 0.0 0.0 setrgbcolor\n");
  fprintf(epsf, "  /r exch def /y exch def /x exch def\n");
  fprintf(epsf, "  newpath x r add y moveto x y r 0 359 arc closepath stroke\n");
  fprintf(epsf, "  x r 2.0 mul add y moveto show\n");
  fprintf(epsf, "  } def\n");
  for(i=0; i<ns; i++) {
    if (targetx==xs[i] && targety==ys[i])
      fprintf(epsf, "1 0 0 setrgbcolor\n");
    else {
      fprintf(epsf, "0 0 0 setrgbcolor\n");
      for(j=0; j<nrefs; j++)
	if(refx[j]==xs[i] && refy[j]==ys[i]) {
	  fprintf(epsf, "0 0 1 setrgbcolor\n");
	}
    }
    fprintf(epsf, "%f %f %f picstar\n", xs[i]*scalef, ys[i]*scalef,
	    (ms[i] > 16.0) ? 0.01 : (16.0 - ms[i] + rdmag) * 2.0 * scalef);
  }
  for(i=0; i<nc; i++) {
    dd = sqrt(catxs[i]*catxs[i] + catys[i]*catys[i]);
    theta0 = atan2(catys[i],catxs[i]);
    theta1 = theta0 + romega;
    if (i!=varst) {
      rcipe_label(catname, i, label, 120);
      fprintf(epsf, "(%s) %f %f %f catstar\n", label,
              catpx[i] = (dd*rampx*cos(theta1)+rdx)*scalef,
	      catpy[i] = (dd*rampy*sin(theta1)+rdy)*scalef,
	      (catms[i] > 18.0) ? 0.01 : (17.0 - catms[i]) * 2.0 * scalef);
      strcpy(label, "?");
    } else {
      vx = (dd*rampx*cos(theta1)+rdx)*scalef;
      vy = (dd*rampy*sin(theta1)+rdy)*scalef;
    }
  }
  fprintf(epsf, "0.0 0.5 0.0 setrgbcolor\n");
  fprintf(epsf, "newpath %f %f moveto 0 5 rmoveto 0 10 rlineto stroke\n",vx,vy);
  fprintf(epsf, "newpath %f %f moveto 5 0 rmoveto 10 0 rlineto stroke\n",vx,vy);
  fprintf(epsf, "newpath %f %f moveto 0 -5 rmoveto 0 -10 rlineto stroke\n",vx,vy);
  fprintf(epsf, "newpath %f %f moveto -5 0 rmoveto -10 0 rlineto stroke\n",vx,vy);
  fclose(epsf);
}

char * aavso_results(char obsid[],
		     char *objname,
		     double obstime,
		     char magflag,
		     float obsmag,
		     char obscmt[],
		     char obscomp[],
		     char obscharts[],
		     char obscode[],
		     char obsrem[]) {

  int i;

  static char aares[100];

  for(i=0; objname[i]; i++)
    if(objname[i]=='-') objname[i] = ' ';

  if(obsmag>15.5)
      sprintf(aares, "%-8s%-10s%12.4f<15.50%-7s%-11s%-8s%-5s%-80s",
	      obsid,objname,obstime,obscmt,obscomp,obscharts,obscode,obsrem);
    else
      sprintf(aares, "%-8s%-10s%12.4f%c%5.2f%-7s%-11s%-8s%-5s%-80s",
	      obsid,objname,obstime,magflag,
	      obsmag,obscmt,obscomp,obscharts,obscode,obsrem);
  return aares;
}

char *fits_header(struct ccd_frame *frr, char *name) {
  FITS_row *hr;
  static char r[100];
  int i;

  hr = fits_keyword_lookup(frr,name);
  if (hr==NULL) return "";
  strncpy(r,(char *)hr,80);
  for(i=79; i>0; i--)
    if(r[i]==' ') {
      r[i] = 0;
      break;
    }
  return r;
}

char *fits_val(struct ccd_frame *ffr, char *name) {
  static char r[100];
  char *s;
  char *t;

  t = r;
  s = fits_header(ffr, name);
  while(*s && (*s != '=')) s++;
  if (*s == '=') s++;
  while(*s && (*s != '/')) {
    if((*s != ' ') && (*s != '\''))
      *t++ = *s;
    s++;
  }
  *t = 0;
  return r;
}

typedef struct xym {
  float x,y,m;
} XYM;

XYM sexs[10000];

int xymcomp(const void *a, const void *b) {
  if (((XYM *)a)->m > ((XYM *)b)->m)
    return -1;
  if (((XYM *)a)->m == ((XYM *)b)->m)
    return 0;
  return 1;
}

int read_sexcat(char *sexcat, float *xs, float *ys, float *ms, int maxn) {


  FILE *f;
  char buf[160];
  int nu, flags;
  float erriso, errau;
  int i,j;

  f = fopen(sexcat,"r");
  if (f==NULL) {
    fprintf(stderr, "Could not open file %s\n");
    exit(-1);
  }
  i=0;
  while (1) {
    fgets(buf,160,f);
    if(feof(f))
      break;
    if (buf[0]!='#') {
      sscanf(buf, "%d%f%f%f%f%f%d",&nu,&erriso,&(sexs[i].m),&errau,&(sexs[i].x),
	     &(sexs[i].y),&flags);
      if(!flags) i++;
    }
  }
  fclose(f);
  qsort(sexs,i,sizeof(XYM),xymcomp);
  for(j=0; j<i && j<maxn; j++) {
    xs[j] = sexs[j].x;
    ys[j] = sexs[j].y;
    ms[j] = -2.511886 * log10(sexs[j].m);
  }
  return i;
}
  
main(short argc, char **argv) {

  int i,j;
  double obst;
  char targ[129];
  float xref, yref, xrefpix, yrefpix, xinc, yinc, rot;
  int matchpicf = 0;
  int buletinf = 0;
  int tabf = 0;
  char matchpicn[128] = "x.eps";
  char buletinn[128] = "x.txt";
  char sexcat_name[128] = "";

  float theta0, theta1;
  float dd, xx, yy, vx, vy;
  float ndist, ddist;

  float estimate;
  double snr = 10.0;
  float degperpix_correction = 1.0;

  FILE *cxrf, *bf;

  int just_list_picture_sources = 0;
  int just_list_catalog_sources = 0;

  int ref_band = V_BAND;

  int correct_to_johnson = 0;
  double johnson[10];  // only first 3 used

  char label[128] = "?";
  char gscl[128] = "?";
  char gsc2l[128] = "?";
  char cmt[200] = "";

  while ((argc>2) && (argv[1][0]=='-')) {
    if (argv[1][1]=='P') { // picture requested
      matchpicf = 1;
      strcpy(matchpicn,argv[1]+2);
      argv ++;
      argc --;
    }
    if (argv[1][1]=='b') { // picture requested
      buletinf = 1;
      strcpy(buletinn,argv[1]+2);
      argv ++;
      argc --;
    }
    if (argv[1][1]=='c') { // comment specified
      strcpy(cmt, argv[1]+2);
      argv ++;
      argc --;
    }
    if (argv[1][1]=='T') { // show tables of matches
      tabf = 1;
      argv ++;
      argc --;
    }
    if (argv[1][1]=='R') { // provide s/n ratio
      snr = atof(argv[1]+2);
      argv++;
      argc--;
    }
    if (argv[1][1]=='s') { // slight degperpix scale factor
      degperpix_correction = atof(argv[1]+2);
      argv++;
      argc--;
    }
    if (argv[1][1]=='C') { // use this sextractor catalog
      strncpy(sexcat_name, argv[1]+2,128);
      argv++;
      argc--;
    }
    if (argv[1][1]=='N') { // observer code
      strncpy(obsc, argv[1]+2,4);
      argv++;
      argc--;
    }

    if (argv[1][1]=='t') { // use tycho reference stars
      ref_band = VT_BAND;
      if (argv[1][2]=='j') {
	correct_to_johnson = 1;
      }
      argv++;
      argc--;
    }

    if (argv[1][1]=='l' && argv[1][2]=='p') {
      just_list_picture_sources = 1;
      argv++;
      argc--;
    }
    if (argv[1][1]=='l' && argv[1][2]=='c') {
      just_list_catalog_sources = 1;
      argv++;
      argc--;
    }
      
  }

  if (argc!=3) {
    fprintf(stderr, "Usage: avsomat [-Ppicname] [-bbuletin] [-T] [-Rsnr] [-lc|-lp] [-t]\n");
    fprintf(stderr, "               [-NOBS] [-Csex.cat] [-ccmt] image.fits recipe.rcp\n");
    exit(-1);
  }

  fr = read_fits_file(argv[1]);
  
  if (fr==NULL) {
    fprintf(stderr, "No such file: %s\n", argv[1]);
    exit(-1);
  }  

  if (!sexcat_name[0]) {
    src.s = calloc(100, sizeof(struct star));
    if (src.s == NULL) {
      fprintf(stderr, "Malloc of src failed\n");
      exit(-1);
    }
    
    src.maxn=100; 
    src.ns = 0;
    ns = extract_stars(fr, NULL, (double)0.0, snr, &src);
    
    for(i=0; i<ns; i++) {
      xs[i] = src.s[i].x;
      ys[i] = src.s[i].y;
      ms[i] = flux_to_absmag(src.s[i].flux);
    }
  } else {
    ns = read_sexcat(sexcat_name,xs,ys,ms,10000);
  }

  if(just_list_picture_sources) {
    printf("# FILENAME= %s\n", argv[1]);
    printf("# SIGMAS  = %f\n", snr);
    printf("# %s (%f)\n", fits_header(fr,"NAXIS1"), fr->geom.sz_x);
    printf("# %s\n", fits_header(fr,"NAXIS2"));
    printf("# %s\n", fits_header(fr,"EXPTIME"));
    printf("# %s\n", fits_header(fr,"FILTER"));
    printf("# %s\n", fits_header(fr,"OBJECT"));
    printf("# %s\n", fits_header(fr,"DATE-OBS"));
    printf("# %s\n", fits_header(fr,"JDATE"));
    printf("# %s\n", fits_header(fr,"CRPIX1"));
    printf("# %s\n", fits_header(fr,"CRPIX2"));
    printf("# %s\n", fits_header(fr,"CDELT1"));
    printf("# %s\n", fits_header(fr,"CDELT2"));
    printf("# %s\n", fits_header(fr,"CROTA1"));
    printf("# %s\n", fits_header(fr,"CRVAL1"));
    printf("# %s\n", fits_header(fr,"CRVAL2"));

    for(i=0; i<ns; i++) {
      printf("%8.1f %8.1f %8.2f\n", xs[i], ys[i], ms[i]);
    }
    exit(0);
  }


  fr -> fim.xinc *= degperpix_correction;
  fr -> fim.yinc *= degperpix_correction;

  xref = fr -> fim.xref;
  yref = fr -> fim.yref;
  xrefpix = fr -> fim.xrefpix;
  yrefpix = fr -> fim.yrefpix;
  xinc = fr -> fim.xinc;
  yinc = fr -> fim.yinc;
  rot = fr -> fim.rot;

  strcpy(targ, argv[1]);
  obst = atof(fits_header(fr,"JDATE")+14);

  //  ns = ip_lextract(argv[1],snr,60000,xs,ys,ms,10000,targ,129,&obst,
  //		   &xref, &yref, &xrefpix, &yrefpix, &xinc, &yinc, &rot);

  if(tabf) {
    printf("%d sources for %s field at %f\n",ns,targ,obst);
    printf("xref=%f yref=%f\nxrefpix=%f yrefpix=%f\nxinc=%.10f yinc=%.10f rot=%f\n",
  	 xref, yref, xrefpix, yrefpix, xinc, yinc, rot);
    for(i=0; i<ns; i++)
      printf("%3d. %7.1f %7.1f    %5.2f\n",i,xs[i],ys[i],ms[i]);
  }

  
  nc = rcipe_xy(argv[2], catxs, catys, xinc, 10000);
  rcipe_magnitudes(argv[2], P_BAND, catms, 10000); // P_BAND is for orientation
  rcipe_magnitudes(argv[2], FG_BAND, catfgms, 10000); // P_BAND is for orientation
  rcipe_magnitudes(argv[2], JG_BAND, catjgms, 10000); // P_BAND is for orientation

  for(i=0; i<nc; i++)
    if(catms[i]<-100) {
      if (catfgms[i]>-100) catms[i] = catfgms[i];
      if (catjgms[i]>-100) catms[i] = catjgms[i];
      if (catms[i]<-100)
	catms[i] = catms[0];
    }

  if(tabf) {
    printf("Got %d catalog entries\n",nc);
    for(i=0; i<nc; i++)
      printf("%3d. %7.1f %7.1f    %5.2f\n",i,catxs[i],catys[i],catms[i]);
  }

  if(just_list_catalog_sources) {
    printf("# cat %s\n", argv[2]);
    printf("# dpp %f\n", xinc);
    for(i=0; i<nc; i++) {
      printf("%8.1f %8.1f %8.2f\n", catxs[i], catys[i], catms[i]);
    }
    exit(0);
  }

  nr = starmatch(xs, ys, ms, (ns>50) ? 50 : ns,
		 catxs, catys, catms, nc,
		 matches, &romega, &rdx, &rdy, &rampx, &rampy, &rdmag);

  xinc = (fr -> fim.xinc = fr -> fim.xinc * 2.0/(rampx+rampy));
  yinc = (fr -> fim.yinc = fr -> fim.yinc * 2.0/(rampx+rampy));

  nc = rcipe_xy(argv[2], catxs, catys, xinc, 10000);

  rcipe_magnitudes(argv[2], P_BAND, catms, 10000); // P_BAND is for orientation
  rcipe_magnitudes(argv[2], FG_BAND, catfgms, 10000); // P_BAND is for orientation
  rcipe_magnitudes(argv[2], JG_BAND, catjgms, 10000); // P_BAND is for orientation

  for(i=0; i<nc; i++)
    if(catms[i]<-100) {
      if (catfgms[i]>-100) catms[i] = catfgms[i];
      if (catjgms[i]>-100) catms[i] = catjgms[i];
      if (catms[i]<-100)
	catms[i] = catms[0];
    }

  nr = starmatch(xs, ys, ms, (ns>50) ? 50 : ns,
		 catxs, catys, catms, nc,
		 matches, &romega, &rdx, &rdy, &rampx, &rampy, &rdmag);

  if (verbosef) {
    printf("#   Found %d matches\n", nr);
    printf("#   omega=%f,dx=%f,dy=%f,ampx=%f,ampy=%f,dmag=%f\n",
	   romega, rdx, rdy, rampx, rampy, rdmag);
  }

  if (nr<3) {
    printf("%s failed\n", argv[1]);
    exit(-1);
  }

  if(tabf) {
    printf("After match %d catalog entries\n",nc);
    for(i=0; i<nc; i++)
      printf("%3d. %7.1f %7.1f    %5.2f\n",i,catxs[i],catys[i],catms[i]);
    printf("PLATE - CATLG\n");
    for(i=0; i<ns; i++)
      if(matches[i]!=-1)
	printf("%5d (%f,%f:%f) - %5d (%f,%f:%f) \n",i,xs[i],ys[i],ms[i]-rdmag,
	       matches[i],catxs[matches[i]],catys[matches[i]],
	       catms[matches[i]]);
  }

  // compute plate positions of the shorcat stars

  for(i=0; i<nc; i++) {
    dd = sqrt(catxs[i]*catxs[i] + catys[i]*catys[i]);
    theta0 = atan2(catys[i],catxs[i]);
    theta1 = theta0 + romega;
    catpx[i] = dd*rampx*cos(theta1)+rdx;
    catpy[i] = dd*rampy*sin(theta1)+rdy;
  }

  // identify reference stars; 

  rcipe_magnitudes(argv[2], ref_band, phms, 10000);
  if (correct_to_johnson && (ref_band==VT_BAND))
    rcipe_magnitudes(argv[2], BT_BAND, bt_phms, 10000);

  rcipe_varstars(argv[2],varstars,1000); // only the first will be considered

  nrefs = rcipe_refs(argv[2],varstars[0],10.0,ref_band,refids,100);

  targetx = catpx[varstars[0]];
  targety = catpy[varstars[0]];

  // correction: if a plate source is very close to targetx/targety take that
  //             as targetx/targety

  ddist = rcipe_postol(argv[2],varstars[0]) / fabs(xinc * 3600.0);

  if (ddist<0.0)
    ddist = 5.0;

  for (i=0; i<ns; i++) {
    if ((ndist = sqrt((xs[i]-targetx)*(xs[i]-targetx) + (ys[i]-targety)*(ys[i]-targety)))
	<ddist) {
      if (verbosef)
	printf("# correcting target (%.1f:%.1f) into (%.1f:%.1f)\n",
	       targetx,targety,xs[i],ys[i]);
      ddist = ndist;
      targetx = xs[i];
      targety = ys[i];
    }
  }

  for (j=0; j<nrefs && j<3; j++) { // at most 3 references in aavso bulletins
    refx[j] = catpx[refids[j]];
    refy[j] = catpy[refids[j]];
    refm[j] = phms[refids[j]]; // the V magnitudes
    if(correct_to_johnson) {
      refm[j] = refm[j] - 0.09 * (bt_phms[refids[j]]-phms[refids[j]]);
      if (verbosef)
	fprintf(stderr, "VT=%f, V=%f (B-V)T=%f\n", 
		phms[refids[j]], refm[j], bt_phms[refids[j]]-phms[refids[j]]);
    }

    ddist = 5.0;
    for (i=0; i<ns; i++) {
      if ((ndist = sqrt((xs[i]-refx[j])*(xs[i]-refx[j]) + (ys[i]-refy[j])*(ys[i]-refy[j])))
	  <ddist) {
	if (verbosef)
	  printf("# correcting reference %.1f (%.1f:%.1f) into (%.1f:%.1f)\n",
		 refm[j],refx[j],refy[j],xs[i],ys[i]);
	ddist = ndist;
	refx[j] = xs[i];
	refy[j] = ys[i];
      }
    }
  }

  nrefs = j;

  if (matchpicf)
    sketch_match(matchpicn, 2*(int)xrefpix, 2*(int)yrefpix, varstars[0], argv[2]);

  if (verbosef)
    printf("#   ");
  estimate = 0.0;


  cxrf = fopen("default.cxr", "r");
  if (cxrf==NULL) {
    fprintf(stderr, "Cannot open default.cxr\n");
    exit(-1);
  }
  fscan_recipy(cxrf, &vs);
  fclose(cxrf);

  for(i=0; i<nrefs; i++) {
    add_std_star(&vs, refx[i], refy[i], refm[i]);
  }
  add_pgm_star(&vs, targetx, targety);

  measure_stars(fr, &vs);
  get_ph_solution(&vs);

  rcipe_varid(argv[2],varstars[0],obji,30);
  rcipe_label(argv[2],varstars[0],objn,30);
  rcipe_chart(argv[2],varstars[0],chrt,30);

  if(nrefs==1)
    sprintf(objc,"%d",(int)(refm[0]*10.0));
  if(nrefs==2)
    sprintf(objc,"%d,%d",(int)(refm[0]*10.0),(int)(refm[1]*10.0));
  if(nrefs==3)
    sprintf(objc,"%d,%d,%d",(int)(refm[0]*10.0),
	    (int)(refm[1]*10.0),(int)(refm[2]*10.0));

  if (ref_band==V_BAND)
    printf("\n%s\n",aavso_results(obji,objn,obst,' ',vs.s[nrefs].stdmag,
				  " CCDV",objc,chrt,obsc,cmt));
  if (ref_band==VT_BAND && !correct_to_johnson)
    printf("\n%s\n",aavso_results(obji,objn,obst,' ',vs.s[nrefs].stdmag,
				  " T CCDV",objc,"AVSOMAT",obsc,cmt));
  if (ref_band==VT_BAND && correct_to_johnson)
    printf("\n%s\n",aavso_results(obji,objn,obst,' ',vs.s[nrefs].stdmag,
				  " K CCDV",objc,"AVSOMAT",obsc,cmt));


  if(buletinf) {
    bf = fopen(buletinn, "w");

    fprintf(bf, "Result in AAVSO format:\n");
    if (ref_band==V_BAND)
      fprintf(bf, "\n%s\n",aavso_results(obji,objn,obst,' ',vs.s[nrefs].stdmag,
					 " CCDV",objc,chrt,obsc,""));
    if (ref_band==VT_BAND && !correct_to_johnson)
      fprintf(bf, "\n%s\n",aavso_results(obji,objn,obst,' ',vs.s[nrefs].stdmag,
					 " T CCDV",objc,"AVSOMAT",obsc,""));
    if (ref_band==VT_BAND && correct_to_johnson)
      fprintf(bf, "\n%s\n",aavso_results(obji,objn,obst,' ',vs.s[nrefs].stdmag,
					 " K CCDV",objc,"AVSOMAT",obsc,""));

    if (ref_band==V_BAND)
      fprintf(bf, "\n\nUsing AAVSO reference stars:\n\n");
    if (ref_band==VT_BAND && !correct_to_johnson)
      fprintf(bf, "\n\nUsing uncorrected Tycho-VT reference stars:\n\n");
    if (ref_band==VT_BAND && correct_to_johnson)
      fprintf(bf, "\n\nUsing Tycho-VT reference stars, transformed to Johnson:\n\n");
    fprintf(bf, "%4s %6s %15s %15s\n", "NAME", "MAG", "GSC-1 ID", "GSC-2 ID");
    for(i=0; i<nrefs; i++) {
      rcipe_label(argv[2], refids[i], label, 120);
      rcipe_gsclink(argv[2], refids[i], gscl, 120);
      rcipe_gsc2link(argv[2], refids[i], gsc2l, 120);
      fprintf(bf, "%-4s %6.2f %15s %15s\n", label, refm[i], gscl, gsc2l);
    }

    fprintf(bf, "\n\nFITS HEADER:\n", snr);

    fprintf(bf, "\nRESOLUTION (bit/pixel): %s\n", fits_val(fr,"BITPIX"));
    fprintf(bf, "IMAGE SIZE: %s ", fits_val(fr,"NAXIS1"));
    fprintf(bf,    "x %s\n", fits_val(fr,"NAXIS2"));
    fprintf(bf, "DEG PER PIXEL: %s ", fits_val(fr,"CDELT1"));
    fprintf(bf,    "x %s\n", fits_val(fr,"CDELT2"));
    fprintf(bf, "CCD BINNING: %s ", fits_val(fr,"CCDBIN1"));
    fprintf(bf,    "x %s\n", fits_val(fr,"CCDBIN2"));
    fprintf(bf, "SENSOR TEMP (K): %s\n", fits_val(fr,"SNS_TEMP"));
    fprintf(bf, "EXP TIME (s): %s\n", fits_val(fr,"EXPTIME"));


    fprintf(bf, "\nOBJECT: %s\n", fits_val(fr,"OBJECT"));
    fprintf(bf, "JDATE EXP START: %s\n", fits_val(fr,"JDATE"));
    fprintf(bf, "RA: %s\n", fits_val(fr,"OBJCTRA"));
    fprintf(bf, "DEC: %s\n", fits_val(fr,"OBJCTDEC"));
    fprintf(bf, "EQUINOX: %s\n", fits_val(fr,"EQUINOX"));

    fprintf(bf, "\nOBSERVER: %s\n", fits_val(fr, "OBSERVER"));
    fprintf(bf, "LATITUDE: %s\n", fits_val(fr, "LAT-OBS"));
    fprintf(bf, "LONGITUDE: %s\n", fits_val(fr, "LONG-OBS"));
    fprintf(bf, "ALTITUDE: %s\n", fits_val(fr, "ALT-OBS"));

    fprintf(bf, "\nINSTRUMENT: %s\n", fits_val(fr,"INSTRUME"));
    fprintf(bf, "FILTER: %s\n", fits_val(fr,"FILTER"));
    fprintf(bf, "TELESCOPE: %s\n", fits_val(fr,"TELESCOP"));
    fprintf(bf, "FOCUS: %s\n", fits_val(fr,"FOCUS"));
    fprintf(bf, "FOCAL LENGTH (cm): %s\n", fits_val(fr,"FLEN"));
    fprintf(bf, "APERTURE (cm): %s\n", fits_val(fr,"APERT"));

  }
}
